//+------------------------------------------------------------------+
//|                                                RaptorEA_v1.03.mq5 |
//|                                                  Abiodun Farinre |
//|                                            abbeycity10@yahoo.com |
//+------------------------------------------------------------------+
#property copyright "Abiodun Farinre"
#property link      "abbeycity10@yahoo.com"
#property version   "1.03"
//+------------------------------------------------------------------+
//| Description                                                      |
//+------------------------------------------------------------------+
#property description "This EA is based on KNN classifier."
#property description "Uses Price and a few indicators as inputs"
#property description "(Bollinger bands as trigger)."
#property description "Automatic Learning form past data. "
//+------------------------------------------------------------------+
//| Includes                                                         |
//+------------------------------------------------------------------+
#include <Trader.mqh>
//+------------------------------------------------------------------+
//| Input Variables                                                  |
//+------------------------------------------------------------------+
input double   Lots                    =  0.5;     //Volume
input int      Stop_Loss               =  660;      //Stop Loss
input int      Take_Profit             =  0;      //Take Profit
input int      Trailing_Stop           =  165;
input int      Bolinger_Period         =  32;
input double   Bolinger_Deviation      =  2.8;
input int      Dema_Period             =  63;
input int      Fast_EMA                =  12;
input int      Slow_EMA                =  26;
input int      MACD_SMA                =  9;
input int      MA_Period               =  14;

//+------------------------------------------------------------------+
//| KNN Class                                                        |
//+------------------------------------------------------------------+
class KNN : public Trader {
   private:
      MqlRates       EurUsdRates[]; 
      MqlRates       GbpUsdRates[];
      MqlRates       EurGbpRates[];
      
      ushort         MaxNearestNeighbours;
      bool           ExactMatch;
      double         BuyOpenPrice;
      double         SellOpenPrice;
      int            LastTrainedBar;
      int            LastTradedBar;
      bool           Train;
      bool           Trade;
      
      double         NewData[];
      
      int            Bolinger_EURUSD_Handle;
      int            DEMA_EURUSD_Handle;
      int            MACD_EURUSD_Handle;
      
      int            MA_EURUSD_Handle;
      int            MA_GBPUSD_Handle;
      int            MA_EURGBP_Handle;
      
      double         Bolinger_Top[];
      double         Dema[];
      double         MACD_EURUSD[];
      
      double         Bolinger_Bottom[];

      double         Signal_EURUSD[];
      
      double         MA_EURUSD[];
      double         MA_EURGBP[];
      double         MA_GBPUSD[];
      
      double         Max[]; 
      double         Min[];
   
   public:
      
      void           KNN();
      void           ~KNN();      
      int            GetClass(double &TestData[]);
      void           SetMaxNearestNeighbours(ushort MaxNearestNeighbours){this.MaxNearestNeighbours = MaxNearestNeighbours;};
      bool           TickEvent();
      void           SetTrade(bool Flag){Trade = Flag;};
      void           SaveData();
   protected:
      double         TrainingData[][46];  // 0 - CLASS| 1 - 45 - DATA
      void           SetTrain(bool Flag){Train = Flag;};
      void           LoadData();
      void           NormalizeData(double &IndicatorValues[]);
      bool           AddNewData(char Class, double &NewData[]);
      bool           GetIndicatorValue(double &IndicatorValues[]);
      
      
      
};
//+------------------------------------------------------------------+
//| Class Constructor                                                |
//+------------------------------------------------------------------+
void KNN::KNN(){
   ArrayResize(TrainingData, 0);
   ExactMatch = false;
   SetMaxNearestNeighbours(10);
   SetShowAlert(false);
   
   Bolinger_EURUSD_Handle  = iBands("EURUSD",PERIOD_M15,Bolinger_Period,0,Bolinger_Deviation,PRICE_CLOSE);;
   DEMA_EURUSD_Handle      = iDEMA("EURUSD",PERIOD_M30,Dema_Period,0,PRICE_CLOSE);
   MACD_EURUSD_Handle      = iMACD("EURUSD", PERIOD_M15, Fast_EMA,Slow_EMA, MACD_SMA,PRICE_OPEN);
   
   MA_EURUSD_Handle = iMA("EURUSD", PERIOD_M10, MA_Period, 0, MODE_EMA, PRICE_OPEN);
   MA_GBPUSD_Handle = iMA("GBPUSD", PERIOD_M10, MA_Period, 0, MODE_EMA, PRICE_OPEN);
   MA_EURGBP_Handle = iMA("EURGBP", PERIOD_M10, MA_Period, 0, MODE_EMA, PRICE_OPEN);
   
   double TempMax[] = {0.02873500000000062, 0.02913500000000058, 0.02913500000000058, 0.02913500000000058, 0.02913500000000058, 0.06130331293708591, 0.07099457230084449, 0.07592051991916637, 0.0812259066346892, 0.09241470996700696, 0.004848498927344735, 0.004848498927344735, 0.004848498927344735, 0.004848498927344735, 0.004848498927344735, 0.009601063672687982, 0.01130348735714382, 0.01130348735714382, 0.01130348735714382, 0.01130348735714382, 0.0196622456217499, 0.0196622456217499, 0.0196622456217499, 0.0196622456217499, 0.0196622456217499, 0.01244559386636435, 0.01244559386636435, 0.01244559386636435, 0.01244559386636435, 0.01244559386636435, 0.02119999999999989, 0.02119999999999989, 0.02119999999999989, 0.02119999999999989, 0.02119999999999989, 0.02369999999999983, 0.02369999999999983, 0.02369999999999983, 0.02369999999999983, 0.02369999999999983, 0.01219999999999999, 0.01219999999999999, 0.01219999999999999, 0.01219999999999999, 0.01219999999999999};
   double TempMin[] = {-0.03048999999999991, -0.0306099999999998, -0.0306099999999998, -0.0306099999999998, -0.0306099999999998, -0.09290191760404953, -0.1161491534754655, -0.1381209939336168, -0.1562680788818445, -0.1701024061578587, -0.005667236165921229, -0.005667236165921229, -0.005667236165921229, -0.005667236165921229, -0.005667236165921229, -0.05637564217777946, -0.05529246738505145, -0.05529246738505145, -0.05529246738505145, -0.05529246738505145, -0.03136948441321907, -0.03136948441321907, -0.03136948441321907, -0.03136948441321907, -0.03136948441321907, -0.01403044021247668, -0.01403044021247668, -0.01403044021247668, -0.01403044021247668, -0.01403044021247668, -0.02139999999999986, -0.02139999999999986, -0.02139999999999986, -0.02139999999999986, -0.02139999999999986, -0.02479999999999993, -0.02479999999999993, -0.02479999999999993, -0.02479999999999993, -0.02479999999999993, -0.0121, -0.0121, -0.0121, -0.0121, -0.0121};
 
   if(Bolinger_EURUSD_Handle == INVALID_HANDLE) ReportError("Error getting EURUSD Bolinger Indicator");
   if(DEMA_EURUSD_Handle     == INVALID_HANDLE) ReportError("Error getting EURUSD DEMA Indicator");
   if(MACD_EURUSD_Handle     == INVALID_HANDLE) ReportError("Error getting EURUSD MACD Indicator");
   if(MA_EURUSD_Handle       == INVALID_HANDLE) ReportError("Error getting EURUSD MA Indicator");
   if(MA_GBPUSD_Handle       == INVALID_HANDLE) ReportError("Error getting GBPUSD MA Indicator");
   if(MA_EURGBP_Handle       == INVALID_HANDLE) ReportError("Error getting EURGBP MA Indicator");
   
   ArrayResize(Max, (ArrayRange(TrainingData, 1)-1));
   ArrayResize(Min, (ArrayRange(TrainingData, 1)-1));
   
   ArrayCopy(Max, TempMax);
   ArrayCopy(Min, TempMin);

   LoadData();
   SetTrain(true);

}
//+------------------------------------------------------------------+
//| Class Destructor                                                 |
//+------------------------------------------------------------------+
void KNN::~KNN(){
   IndicatorRelease(MACD_EURUSD_Handle);
   IndicatorRelease(DEMA_EURUSD_Handle);
   IndicatorRelease(Bolinger_EURUSD_Handle);
   
   IndicatorRelease(MA_EURUSD_Handle);
   IndicatorRelease(MA_GBPUSD_Handle);
   IndicatorRelease(MA_EURGBP_Handle);
   
   SaveData();
}
//+------------------------------------------------------------------+
//| TickEvent Function                                               |
//+------------------------------------------------------------------+
bool KNN::TickEvent(){
   if(LastTrainedBar+2 < Bars(_Symbol,PERIOD_M15)){
      int mod = 1;
      
      if(GetDigits()==5 || GetDigits()==3){
         mod = mod*10;
      }
      
      MqlTick TickData;
      MqlRates Rates[];
      
      
      ArraySetAsSeries(Bolinger_Top, true);
      ArraySetAsSeries(Bolinger_Bottom, true);
      ArraySetAsSeries(Dema, true);
      ArraySetAsSeries(Rates,true);
      
      
      if(!SymbolInfoTick("EURUSD",TickData)){
         ReportError("SymbolInfoTick() failed, error = ",GetLastError());
      }
      
      if(CopyRates("EURUSD",PERIOD_M30,0,3,Rates)<0){
         ReportError("Error copying history data, error = ",GetLastError());
         return(false);
      }
      
      if(CopyBuffer(Bolinger_EURUSD_Handle,1,0,3,Bolinger_Top)<0 || CopyBuffer(Bolinger_EURUSD_Handle,2,0,3,Bolinger_Bottom)<0){
         ReportError("Error copying Bolinger EURUSD indicator buffer, error = ",GetLastError());
         return(false);
      }
      
      if(CopyBuffer(DEMA_EURUSD_Handle,0,0,3,Dema)<0){
         ReportError("Error copying DEMA indicator buffer error = ",GetLastError());
         return(false);
     }
     
      
      
      
      
      
      bool BUY       =(Rates[1].close > Bolinger_Bottom[1] && Rates[1].open < Bolinger_Bottom[1] /*&& Dema[0]>Dema[1] && Dema[1]>Dema[2]*/);

      bool SELL      =(Rates[1].close < Bolinger_Top[1] && Rates[1].open > Bolinger_Top[1]/* && Dema[0]<Dema[1] && Dema[1]<Dema[2]*/);

      bool CLOSE_BUY =(Rates[1].close<Bolinger_Top[1] && Rates[1].open>Bolinger_Top[1]);      

      bool CLOSE_SELL=(Rates[1].close>Bolinger_Bottom[1] && Rates[1].open<Bolinger_Bottom[1]);  



      double Data[];
      /////////////////////////////////////
      if(BuyOpenPrice <=0 && SellOpenPrice <= 0){
         BuyOpenPrice = TickData.ask;
      }
      

      if(CLOSE_BUY){
         if(PositionSelect(_Symbol)){
            if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY){         
               CloseTrade(PositionGetInteger(POSITION_MAGIC));
            }
         }

         if(Train){ 
            if(ArrayRange(NewData, 0)>0){
               if(((TickData.bid-BuyOpenPrice)/_Point) > 0/*10*mod*/){
                  AddNewData(0, NewData);
               }else if(((SellOpenPrice-TickData.ask)/_Point) > 0/*10*mod*/){
                  AddNewData(1, NewData);
               }else{
                  AddNewData(-1, NewData);
               }
               BuyOpenPrice = -1;
               ArrayResize(NewData, 0);
            }
         }
      }
      ////////////////////////////////////////////////////////
      if(CLOSE_SELL){
         if(PositionSelect(_Symbol)){
            if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL){         
               CloseTrade(PositionGetInteger(POSITION_MAGIC));
            }
         }
         if(Train){
            if(ArrayRange(NewData, 0)>0){
               if(((SellOpenPrice-TickData.ask)/_Point) >= 10*mod){
                  AddNewData(1, NewData);
               }else if(((TickData.bid-BuyOpenPrice)/_Point) >= 10*mod){
                  AddNewData(0, NewData);
               }else{
                  AddNewData(-1, NewData);
               }
               SellOpenPrice = -1;
               ArrayResize(NewData, 0);
            }
         }
      }
      
      
      
      
      
      ///?????????????????????????????????????????????????????
      //Buy Trigger
      if(BUY){
         if(Trade){
            if(GetIndicatorValue(Data)){
               int Class = GetClass(Data);
               if(Class == 0){
                  if(!PositionSelect(_Symbol)){
                     OpenTrade(Lots, ORDER_TYPE_BUY, Stop_Loss, Take_Profit, 99999);
                     LastTradedBar = Bars(_Symbol,_Period);                     
                  } 
               }
            }else{
               ReportError("Error Getting Indicator Values");
            }
         }
      }
      if(SellOpenPrice>0 && BUY){
         LastTrainedBar = Bars(_Symbol,_Period);
         
         if(Train){
            
            //AddBuy
            
            if(GetIndicatorValue(NewData)){
               BuyOpenPrice = TickData.ask;
            }else{
               BuyOpenPrice = -1;
               ReportError("Error Transfering indicator vals");
            }
         }
      }
      ////////////////////////////////////////////////////////////////////
      //Sell Trigger
      if(SELL){
         if(Trade){
            if(GetIndicatorValue(Data)){
              int Class = GetClass(Data);
               if(Class == 1){
                  if(!PositionSelect(_Symbol)){
                     OpenTrade(Lots, ORDER_TYPE_SELL, Stop_Loss, Take_Profit, 99999);
                     LastTradedBar = Bars(_Symbol,_Period);                     
                  }
                  
               }
               
            }else{
               ReportError("Error Getting Indicator Values");
            }
         
         }
      }
      if( BuyOpenPrice > 0 && SELL){

         LastTrainedBar = Bars(_Symbol,_Period);
         
          if(Train){
            
            //Addsell    
            
            if(GetIndicatorValue(NewData)){
               SellOpenPrice = TickData.bid;
            }else{
               SellOpenPrice = -1;
               ReportError("Error Transfering indicator vals");
            }
         }
      }
   }
   
   
   return(true);
}
//+------------------------------------------------------------------+
//| GetIndicatorValue Function                                       |
//+------------------------------------------------------------------+
bool KNN::GetIndicatorValue(double &IndicatorValues[]){
   ArrayResize(IndicatorValues, (ArrayRange(TrainingData, 1)-1));
   
  
   ArraySetAsSeries(EurUsdRates, true);
   ArraySetAsSeries(GbpUsdRates, true);
   ArraySetAsSeries(EurGbpRates, true);
   
   ArraySetAsSeries(Bolinger_Top, true);
   ArraySetAsSeries(Dema, true);
   ArraySetAsSeries(MACD_EURUSD, true);
   
   ArraySetAsSeries(Bolinger_Bottom, true);

   ArraySetAsSeries(Signal_EURUSD, true);
   
   ArraySetAsSeries(MA_EURUSD, true);
   ArraySetAsSeries(MA_GBPUSD, true);
   ArraySetAsSeries(MA_EURGBP, true);
   
   //EuroUsd prices
   if(CopyRates("EURUSD",PERIOD_M30,0,6,EurUsdRates)<0){
      ReportError("Error copying EURUSD history data",GetLastError());
      return(false);
   }
   //GbpUsd prices
   if(CopyRates("GBPUSD",PERIOD_M30,0,6,GbpUsdRates)<0){
      ReportError("Error copying GBPUSD history data",GetLastError());
      return(false);
   }
   //EuroGpb prices
   if(CopyRates("EURGBP",PERIOD_M30,0,6,EurGbpRates)<0){
      ReportError("Error copying GBPUSD history data",GetLastError());
      return(false);
   }
   //Macd 
   if(CopyBuffer(Bolinger_EURUSD_Handle,0,0,5,Bolinger_Top)<0 || CopyBuffer(Bolinger_EURUSD_Handle,1,0,5,Bolinger_Bottom)<0){
      ReportError("Error copying Bolinger_EURUSD indicator buffer",GetLastError());
      return(false);
   }
   if(CopyBuffer(DEMA_EURUSD_Handle,0,0,5,Dema)<0){
      ReportError("Error copying DEMA_EURUSD indicator buffer",GetLastError());
      return(false);
   }
   if(CopyBuffer(MACD_EURUSD_Handle,0,0,5,MACD_EURUSD)<0  || CopyBuffer(MACD_EURUSD_Handle,1,0,5,Signal_EURUSD)<0){
      ReportError("Error copying MACD_EURUSD indicator buffer",GetLastError());
      return(false);
   }
   //ma
   if(CopyBuffer(MA_EURUSD_Handle,0,0,5,MA_EURUSD)<0){
      ReportError("Error copying MA_EURUSD indicator buffer",GetLastError());
      return(false);
   }
   if(CopyBuffer(MA_GBPUSD_Handle,0,0,5,MA_GBPUSD)<0){
      ReportError("Error copying MA_GBPUSD indicator buffer",GetLastError());
      return(false);
   }
   if(CopyBuffer(MA_EURGBP_Handle,0,0,5,MA_EURGBP)<0){
      ReportError("Error copying MA_EURGBP indicator buffer",GetLastError());
      return(false);
   }
      
   for(int k=0; k< 5; k++){
      IndicatorValues[k  ]  = Bolinger_Top[k]-EurUsdRates[k].open;
      IndicatorValues[k+5]  = Dema[k]-EurUsdRates[k+1].open;;
      IndicatorValues[k+10] = MACD_EURUSD[k]-Signal_EURUSD[k];
      
      IndicatorValues[k+15] = EurUsdRates[k].open - Bolinger_Bottom[k];
      IndicatorValues[k+20] = GbpUsdRates[k].open - MA_GBPUSD[k];
      IndicatorValues[k+25] = EurGbpRates[k].open - MA_EURGBP[k];
      
      IndicatorValues[k+30] = EurUsdRates[k].open - EurUsdRates[k+1].open;
      IndicatorValues[k+35] = GbpUsdRates[k].open - GbpUsdRates[k+1].open;
      IndicatorValues[k+40] = EurGbpRates[k].open - EurGbpRates[k+1].open;
      
   } 
   
  // NormalizeData(IndicatorValues);
   //
   return(true);
}
//+------------------------------------------------------------------+
//| NormalizeData Function                                           |
//+------------------------------------------------------------------+
void KNN::NormalizeData(double &IndicatorValues[]){
   
   double TempMax[];
   double TempMin[];
   double TempData[];
   
   ArrayResize(TempMax, ArrayRange(Max, 0));
   ArrayResize(TempMin, ArrayRange(Min, 0));
   ArrayResize(TempData, ArrayRange(IndicatorValues,0));
   
   for(int i=0; i<ArrayRange(IndicatorValues, 0); i++){
      //shifting
      if(Min[i] < 0){
         TempMax[i] = (0-Min[i])+Max[i];
         TempMin[i] = (0-Min[i])+Min[i];
         TempData[i]= (0-Min[i])+IndicatorValues[i];
      }else{
         TempMax[i] = Max[i];
         TempMin[i] = Min[i];
         TempData[i]= IndicatorValues[i];
      }
      //normalizing
      IndicatorValues[i] =  (TempData[i]-TempMin[i])/(TempMax[i]-TempMin[i]);
      
   }
   
   
   return;
}
//+------------------------------------------------------------------+
//| Add New Data Function                                            |
//+------------------------------------------------------------------+
bool KNN::AddNewData(char Class, double &NewData[]){
   
   ExactMatch = false;
   int ReceivedClass = -1;
   
   if(ArrayRange(TrainingData, 0) > MaxNearestNeighbours){
      ReceivedClass = GetClass(NewData);
   }
   
   
   if(ExactMatch == false || ReceivedClass != Class){
      uint row = ArrayRange(TrainingData,0);
      ExactMatch = false;
      ArrayResize(TrainingData, row+1);
      
      TrainingData[row][0] = Class;
      
      uint size = ArraySize(NewData);
     
      for(uint i =0; i< size; i++){
          TrainingData[row][i+1] = NewData[i];
      }
      
      return(true);
   }
   return(false);
}
//+------------------------------------------------------------------+
//| Classify Data Function                                           |
//+------------------------------------------------------------------+
int KNN::GetClass(double &TestData[]){
   
	double kDistance = 0.00;
	double cData[][2];
	bool OutOfLoop = false;
	
	
	int rows = ArrayRange(TrainingData, 0);
	int t = ArrayRange(TestData, 0);
   
   
   ArrayResize(cData, MaxNearestNeighbours);
   ArrayInitialize(cData, 99999999999999);
   
   for(int row = 0; row < rows; row++){
      for(int h =0; h < t; h++){
         double x = TestData[h];
         double y = TrainingData[row][h+1];
         
         kDistance += (MathPow(MathAbs(x - y), 2));
         
      }
      
      kDistance = MathSqrt(kDistance);
      
     
      
      if(kDistance < cData[MaxNearestNeighbours-1][1]){
         bool done = false;
         for(uint i=0; i<MaxNearestNeighbours; i++ ){
            double tData[][2];
            if(i == 0 && kDistance < cData[0][1]){
               
               ArrayCopy(tData, cData, 0, 0);
               cData[i][1] = kDistance;
               cData[i][0] = TrainingData[row][0];
               ArrayCopy(cData,tData,2,0);
               done = true;
            }else if(i == (ArrayRange(cData, 0)-1) && kDistance < cData[i][1]){
               
               cData[i][1] = kDistance;
               cData[i][0] = TrainingData[row][0];
               done = true;
               
            }else if(kDistance < cData[i][1] && kDistance > cData[i-1][1]){
              
               for(uint c=MaxNearestNeighbours-1; c>=i; c--){
                  cData[c][1] = cData[c-1][1];
                  cData[c][0] = cData[c-1][0];
               }
               cData[i][1] = kDistance;
               cData[i][0] = TrainingData[row][0];
               
               done = true;
               
            } 
            if(done){
               ArrayResize(cData, MaxNearestNeighbours);
               done = false;
               break; 
            }
                     
         }
      }
      
      kDistance = 0;
      
   }
   
      
   int BuySignal     = 0;
   int SellSignal    = 0;
   int NoSignal      = 0;
   

   
   if(cData[0][1] <= 0){
      ExactMatch = true;
   }
      
   for(int j=0; j<ArrayRange(cData, 0); j++){
      if(cData[j][0] == 0)          BuySignal++;
      else if(cData[j][0] == 1)     SellSignal++;
      else if(cData[j][0] == -1)    NoSignal++;
   }
   
      
   
   if(ExactMatch && cData[0][0] == 0)return(0);
   if(ExactMatch && cData[0][0] == 1)return(1);
   
   if(BuySignal-SellSignal >=2/*&& (BuySignal - NoSignal) >= 1*/)return(0);
   if(SellSignal-BuySignal >=2/*&& (SellSignal- NoSignal) >= 1*/)return(1);
   
   return(-1);
}
//+------------------------------------------------------------------+
//| Save Data Function                                               |
//+------------------------------------------------------------------+
void KNN::SaveData(void){
   string subfolder="KNN_Data";
   
   int filehandle=FileOpen("raptordata.bin",FILE_WRITE|FILE_BIN );
   
   if(filehandle!=INVALID_HANDLE){
      uint totalWritten = FileWriteArray(filehandle, TrainingData);
      
      if(totalWritten > 0){
         ReportError("Data saved successfully");
      }else{
         ReportError("Data could not be saved!");
         string x = "";
         string h = "Training Data rows = ";
         string l = IntegerToString(ArrayRange(TrainingData,0));
         StringConcatenate(x, h, l);
         ReportError(x);
      }
      
      FileClose(filehandle);
   }
   else ReportError("File open failed, error ",GetLastError());
  
}
//+------------------------------------------------------------------+
//| Load Data Function                                               |
//+------------------------------------------------------------------+
void KNN::LoadData(void){
   string subfolder="KNN_Data";
   string terminal_data_path=TerminalInfoString(TERMINAL_DATA_PATH);
   
   int filehandle=FileOpen("raptordata.bin",FILE_READ|FILE_BIN );
   
   if(filehandle!=INVALID_HANDLE){
      uint TotalRead = FileReadArray(filehandle, TrainingData);
      if(TotalRead > 0){
         string x = "";
         StringConcatenate(x,"Data read Successfully: ",TotalRead);
         ReportError(x);
      }
     FileClose(filehandle); 
   }
   else ReportError("File open failed, error ",GetLastError());
   
}
//+------------------------------------------------------------------+




KNN neuralNetwork;


//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
//---
   neuralNetwork.SetShowAlert(false);
   neuralNetwork.SetPrintToJournal(false);
   neuralNetwork.SetSymbol(_Symbol);
   neuralNetwork.SetDigits(_Digits);
   neuralNetwork.SetPoint(_Point);
   neuralNetwork.SetMaxNearestNeighbours(8);
   neuralNetwork.SetTrade(true);
   
   neuralNetwork.SetTimeframe(PERIOD_M30);

   neuralNetwork.SetSlippage(10);
   
   neuralNetwork.SetUseMoneyManagement(true, true, 5);
   
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
//---
   neuralNetwork.TickEvent();
   neuralNetwork.SetTrailingStop(Trailing_Stop,neuralNetwork.GetSymbol(), true);        
  }
//+------------------------------------------------------------------+
double OnTester(){
   neuralNetwork.SaveData();
   return(0);
}
