//+------------------------------------------------------------------+
//|                                                       Trader.mqh |
//|                                                  Abiodun Farinre |
//|                                            abbeycity10@yahoo.com |
//+------------------------------------------------------------------+
#property copyright "Abiodun Farinre"
#property link      "abbeycity10@yahoo.com"
#property version   "1.00"




//+------------------------------------------------------------------+
//| Trader Class function                                            |
//+------------------------------------------------------------------+
class Trader{
   private:
     
      MqlTradeRequest         TradeRequest;  
      MqlTradeResult          TradeResult;    
      MqlTick                 CurrentTick;
      bool                    ShowAlert;
      bool                    PrintToJournal;
      bool                    UseMaxAgression;
      bool                    UseMM;
      double                  Risk;      
      int                     digits;
      double                  point;
      string                  symbol; 
      ENUM_TIMEFRAMES         Timeframe;
      
   public:
      void                    Trader();
      void                    ~Trader();
      void                    SetPrintToJournal(bool Flag){PrintToJournal = Flag;};
      void                    SetShowAlert(bool Flag){ShowAlert = Flag;};
      void                    SetUseMoneyManagement(bool UseMM, bool UseMaxAgression, double Risk){this.UseMM = UseMM; this.UseMaxAgression = UseMaxAgression; this.Risk = Risk;};
      int                     GetDigits(){return(digits);};
      double                  GetPoint(){return(point);};
      string                  GetSymbol(){return(symbol);};
      ENUM_TIMEFRAMES         GetTimeframe(){return(Timeframe);};
      void                    SetDigits(int digits){this.digits = digits;};
      void                    SetPoint(double point){this.point = point;};
      void                    SetSymbol(string symbol){TradeRequest.symbol = symbol; this.symbol = symbol;};
      void                    SetRequestTypeFilling(ENUM_ORDER_TYPE_FILLING TypeFilling){TradeRequest.type_filling = TypeFilling;};
      void                    SetSlippage(ushort slippage){TradeRequest.deviation =slippage;};
      bool                    SetTrailingStop(int TrailingStopValue, string symbol, bool UseTrail= false);
      void                    SetTimeframe(ENUM_TIMEFRAMES Timeframe){this.Timeframe = Timeframe;};
      bool                    OpenTrade(double _Lots, ENUM_ORDER_TYPE OrderType, int _StopLoss, int _TakeProfit, long _Magic_Number, bool ForceLotsUse=false);
      bool                    CloseTrade(long _Magic_Number);
      void                    ReportError(string ErrorDescription, int ErrorCode);
      double                  GetMMVolume(bool UseMaxAgression, double Risk, int _stopLoss);

};

//+------------------------------------------------------------------+
//| Class Constructor function                                       |
//+------------------------------------------------------------------+
void Trader::Trader(void){
   TradeRequest.action           =  TRADE_ACTION_DEAL;
   TradeRequest.type_filling     =  ORDER_FILLING_AON;
   TradeRequest.deviation        =  100;
   TradeRequest.symbol           =  Symbol();
   symbol                        =  Symbol();
   point                         =  Point();
   ShowAlert                     =  true;
   PrintToJournal                =  true;
   digits                        =  4;
   UseMM                         =  false;
}
//+------------------------------------------------------------------+
//| Class Destructor function                                       |
//+------------------------------------------------------------------+
void Trader::~Trader(void){


}
//+------------------------------------------------------------------+
//| SetTrailingStop Event                                            |
//+------------------------------------------------------------------+
bool Trader::SetTrailingStop(int TrailingStopValue, string symbol, bool UseTrail= false){
   
   if(!UseTrail){
      return(false);
   }
   
   bool modifyOrder = false;
   bool  retVal     = false;
   
   if(!SymbolInfoTick(GetSymbol(),CurrentTick)){
         ReportError("Error getting the latest price quote",GetLastError());
      return(false);
   }
   
   if(digits==5 || digits==3){
      TrailingStopValue = TrailingStopValue*10;
   }
   

      if(PositionSelect(GetSymbol())==true){
          if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY){
           
            if(PositionGetDouble(POSITION_PROFIT) >= 0 && (SymbolInfoDouble(GetSymbol(),SYMBOL_BID) - (TrailingStopValue*GetPoint())) > PositionGetDouble(POSITION_SL) && (SymbolInfoDouble(GetSymbol(),SYMBOL_BID) - PositionGetDouble(POSITION_PRICE_OPEN)) > TrailingStopValue*GetPoint()){
               TradeRequest.sl      = NormalizeDouble((CurrentTick.bid - (TrailingStopValue*GetPoint())), GetDigits()+1); 
               modifyOrder          = true;
            }
         }else if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL){
            if(PositionGetDouble(POSITION_PROFIT) >= 0 && CurrentTick.ask + (TrailingStopValue*point) <  PositionGetDouble(POSITION_SL) && (PositionGetDouble(POSITION_PRICE_OPEN) - CurrentTick.ask) > TrailingStopValue*Point()){
               TradeRequest.sl      = NormalizeDouble(CurrentTick.ask + TrailingStopValue*point, digits); 
               modifyOrder          = true;
            }
         }
         if(modifyOrder){
            TradeRequest.action  = TRADE_ACTION_SLTP;
            TradeRequest.tp      = PositionGetDouble(POSITION_TP);


            MqlTradeCheckResult  CheckResult;
  
            if(OrderCheck(TradeRequest,CheckResult)){          
               if(OrderSend(TradeRequest,TradeResult)){
                  retVal = true;
               }else{
                  ReportError("The Modified Order could not be completed", GetLastError());
               }
            }
         }
         
         
      }
   
   TradeRequest.action =  TRADE_ACTION_DEAL;
   return(retVal);
}
//+------------------------------------------------------------------+
//| GetMMVolume function                                             |
//+------------------------------------------------------------------+
double Trader::GetMMVolume(bool UseMaxAgression, double Risk, int _stopLoss){
   int mod = 1;
   double volumeStep = SymbolInfoDouble(symbol, SYMBOL_VOLUME_STEP);
   if(volumeStep == 0.01)mod = 2;
   
   
   if(UseMaxAgression){
      return(NormalizeDouble((((SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE)/10000) * MathSqrt( AccountInfoDouble(ACCOUNT_FREEMARGIN) )))/ 1000, mod) );
   }else{
      return(NormalizeDouble(((Risk * AccountInfoDouble(ACCOUNT_FREEMARGIN)/100) /(SYMBOL_TRADE_TICK_VALUE_LOSS * _stopLoss)), mod));
   }
   return(0.1);
}
//+------------------------------------------------------------------+
//| CloseTrade function                                              |
//+------------------------------------------------------------------+
bool Trader::CloseTrade(long _Magic_Number){
      
      
      if(PositionSelect(symbol)==true){
         if(PositionGetInteger(POSITION_MAGIC) != _Magic_Number)  return(false);
         
         if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY){
            if(OpenTrade(PositionGetDouble(POSITION_VOLUME), ORDER_TYPE_SELL,  0, 0, PositionGetInteger(POSITION_MAGIC), true)){

               ReportError("The Order was closed successfully", GetLastError());
               return(true);
            }else{
               ReportError("The Order could not be Closed!", GetLastError());
            }
         }
         
         if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL){
            if(OpenTrade(PositionGetDouble(POSITION_VOLUME), ORDER_TYPE_BUY,  0, 0, PositionGetInteger(POSITION_MAGIC), true)){

               ReportError("The Order was closed successfully", GetLastError());
               return(true);
            }else{
               ReportError("The Order could not be Closed!", GetLastError());
            }
         }
         
      }
   
   return(false);
}
//+------------------------------------------------------------------+
//| OpenTrade function                                               |
//+------------------------------------------------------------------+
bool Trader::OpenTrade(double _Lots, ENUM_ORDER_TYPE OrderType, int _StopLoss, int _TakeProfit, long _Magic_Number, bool ForceLotsUse=false){
   double Lot = _Lots;
   int mod = 1;
   
   if(!SymbolInfoTick(TradeRequest.symbol,CurrentTick)){
         ReportError("Error getting the latest price quote",GetLastError());
      return(false);
   }
   
   if(digits==5 || digits==3){
      mod = mod*10;
   }
   
   if(!ForceLotsUse && UseMM){
      Lot = GetMMVolume(UseMaxAgression, Risk, _StopLoss*mod);
      if(Lot >= 5){
         Lot =4.99;
      }
   }
   
   
   
   
   TradeRequest.action  = TRADE_ACTION_DEAL;
   TradeRequest.volume = Lot;
   TradeRequest.magic = _Magic_Number; 
   
   
   if(OrderType == ORDER_TYPE_BUY){
      TradeRequest.price   = NormalizeDouble(CurrentTick.ask,digits);
      if(_StopLoss==0){
         TradeRequest.sl   = 0;
      }else{
         TradeRequest.sl   = NormalizeDouble(CurrentTick.ask - _StopLoss*point*mod, digits); 
      }
      if(_TakeProfit==0){
         TradeRequest.tp   = 0;
      }else{
         TradeRequest.tp   = NormalizeDouble(CurrentTick.ask + _TakeProfit*point*mod, digits);  
      }
      TradeRequest.type    = ORDER_TYPE_BUY;
         
   }else if(OrderType == ORDER_TYPE_SELL){
      TradeRequest.price   = NormalizeDouble(CurrentTick.bid,digits);
      if(_StopLoss==0){
         TradeRequest.sl   = 0;
      }else{
         TradeRequest.sl      = NormalizeDouble(CurrentTick.bid + _StopLoss*point*mod, digits); 
      }
      if(_TakeProfit==0){
         TradeRequest.tp   = 0;
      }else{
         TradeRequest.tp      = NormalizeDouble(CurrentTick.bid - _TakeProfit*point*mod, digits);  
      }
      TradeRequest.type    = ORDER_TYPE_SELL;
   }                                 
   
   MqlTradeCheckResult  CheckResult;
  
   if(OrderCheck(TradeRequest,CheckResult)){          
      OrderSend(TradeRequest,TradeResult);
   }
   
   if(TradeResult.retcode==10009 || TradeResult.retcode==10008){
      string output = "An Order has been successfully placed with Ticket#: ";
      StringConcatenate(output, output, TradeResult.order, "!");
      ReportError(output, GetLastError());
      return(true);            
   }else{
      ReportError("The Order could not be completed", GetLastError());
   }
   
   
   return(false);
}
//+------------------------------------------------------------------+
//| ReportError function                                             |
//+------------------------------------------------------------------+
void  Trader::ReportError(string ErrorDescription, int ErrorCode=0){
   string error = "";
   if(ErrorCode != 0){
      StringConcatenate(error, " -> Error Code: ",IntegerToString(ErrorCode));
      
   }
   if(ShowAlert)        Alert(ErrorDescription, error );
   if(PrintToJournal)   Print(ErrorDescription, error );
   ResetLastError();
}


