//+------------------------------------------------------------------+
//|                                                    andrehowe.mq5 |
//|                                                        andrehowe |
//|                                             http://andrehowe.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, Andre Howe"
#property link      "http://andrehowe.com"
#property version   "1.00"

//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
//#include <Expert\Expert.mqh>
#include <Arrays\List.mqh>
#include <Object.mqh>
 
//+------------------------------------------------------------------+
//| Enumerations                                                     |
//+------------------------------------------------------------------+
enum lot_type
  {
   e_fixed,  // Fixed
   e_percent // Percent of deposit
  };
//+------------------------------------------------------------------+
//|   Input parameters                                               |
//+------------------------------------------------------------------+
/*
input string     General        = "=== General parameters ===";
input ushort     PERIOD         =     250; // Calculation period
input ushort     STOP_LOSS      =      50; // The level of stop loss 
input ushort     TAKE_PROFIT    =     100; // The level of Take Profit
input ushort     INSIDE_LEVEL   =      30; // Level back into the channel
input ushort     TRAILING_STOP  =      20; // Trailing Stop Level
input ushort     TRAILING_STEP  =      10; // Trailing Step Level
input ushort     ORDER_STEP     =      10; // Step movement of a pending order
input ushort     SLIPPAGE       =       2; // Slippage
input double     LOT            =       5; // Lot value
input lot_type   LOT_TYPE       =      e_fixed; // Lot type
input bool       LOT_CORRECTION =    true; // Correction of lot size
*/
input string     General        = "=== General parameters ===";
input ushort     PERIOD         =     250; // Calculation period
input ushort     STOP_LOSS      =     100; // The level of stop loss 
input ushort     TAKE_PROFIT    =      50; // The level of Take Profit
input ushort     INSIDE_LEVEL   =      30; // Level back into the channel
input ushort     TRAILING_STOP  =      20; // Trailing Stop Level
input ushort     TRAILING_STEP  =      10; // Trailing Step Level
input ushort     ORDER_STEP     =      10; // Step movement of a pending order
input ushort     SLIPPAGE       =       2; // Slippage
input double     LOT            =     10.0; // Lot value
input lot_type   LOT_TYPE       = e_percent; // Lot type
input bool       LOT_CORRECTION =    true; // Correction of lot size
input ushort     VERBOSE        =      0;  // verbose logging


class CCurrency : public CObject
	{
	
public:
    CCurrency(string symbol,long magic);
   ~CCurrency();
		
   long   magic;
   string symbol;
   
	ulong stop_loss;          // stop loss
	ulong take_profit;        // take profit
	ulong inside_level;       // level back into the channel
	ulong trailing_level;     // trailing stop
	ulong trailing_step;      // trailing step
	ulong order_step;         // order step
	ulong slippage;           // slippage
	double lot;               // lot

	bool buy_open;			     // buy flag
	bool sell_open;			  // sell flag
	double CalcHigh;	        // resistance level
	double CalcLow;   	     // support level
	double High[], Low[];     // High and Low arrays

	MqlTick tick;             // current price
	
	double order_open_price;  // open order price
	double spread;            // spread
	ulong stop_level;         // minimum level of the price to install a Stop Loss / Take Profit
	ulong order_type;         // order type
	ulong order_ticket;       // order ticket 
	
	};
	
//+------------------------------------------------------------------+
//|   Initialization function of the expert                          |
//+------------------------------------------------------------------+
CCurrency::CCurrency(string asymbol,long amagic){
   magic = amagic;
   symbol = asymbol;
//--- Validating input parameters
   if(LOT_TYPE==e_percent && LOT>100)
   {
      // TODO how to fix this - constant value
      //LOT = 100;
   }
//---
   stop_loss      = STOP_LOSS;
   take_profit    = TAKE_PROFIT;
   inside_level   = INSIDE_LEVEL;
   trailing_level = TRAILING_STOP;
   trailing_step  = TRAILING_STEP;
   slippage       = SLIPPAGE;
   order_step     = ORDER_STEP;

//--- If the price includes 3 / 5 decimal places
   long digits = SymbolInfoInteger(symbol,SYMBOL_DIGITS);
   if(digits == 3 || digits == 5)
    {
      stop_loss      = stop_loss      * 10;
      take_profit    = take_profit    * 10;
      inside_level   = inside_level   * 10;
      trailing_level = trailing_level * 10;
      trailing_step  = trailing_step  * 10;
      slippage       = slippage       * 10;
      order_step     = order_step     * 10;
    }
    
	buy_open    = false; // buy flag
	sell_open   = false; // sell flag
	CalcHigh  = 0;     // resistance level
	CalcLow   = 0;     // support level
    
}

CCurrency::~CCurrency(){
}

//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
//CExpert ExtExpert;
CList     *gCurrencyModelList;
CCurrency *gEurUsd;
CCurrency *gUsdChf;
CCurrency *gGbpUsd;
CCurrency *gEurGbp;
CCurrency *gAudUsd;
CCurrency *gEurAud;

class CCurrencyModel : public CObject
	{
public:
	 CCurrencyModel();
	~CCurrencyModel();
	string            ModelName(){return(m_model_name);}
	ENUM_TIMEFRAMES   PeriodA(void){return(m_timeframe_a);}
	ENUM_TIMEFRAMES   PeriodB(void){return(m_timeframe_b);}
	string            SymbolA(void){return(m_ca.symbol);}
	string            SymbolB(void){return(m_ca.symbol);}
	
	int OnInit(string model_name,CCurrency* ca,CCurrency* cb ,ENUM_TIMEFRAMES tf_a,ENUM_TIMEFRAMES tf_b);
	void OnTick();
	void WorkWithPositions(CCurrency* c);
	void WorkWithPendingOrders(CCurrency* c);
	void OpenOrderBuyStop(CCurrency* c);
	void OpenOrderSellStop(CCurrency* c);
	double Calculate_Lot(CCurrency* c,double lot_value,int type,ENUM_ORDER_TYPE direction);
	double NormalizeLot(CCurrency* c,double lot_value);
		
protected:
	MqlTradeRequest request;  // parameters trading query
	MqlTradeResult result;    // result of the trade query
   
   ENUM_TIMEFRAMES   m_timeframe_a;
   ENUM_TIMEFRAMES   m_timeframe_b;
   string            m_model_name;
   
   CCurrency*         m_ca; 
   CCurrency*         m_cb;

	};
	
//+------------------------------------------------------------------+
//|   Initialization function of the expert                          |
//+------------------------------------------------------------------+
CCurrencyModel::CCurrencyModel(){
}

CCurrencyModel::~CCurrencyModel(){
}

int CCurrencyModel::OnInit(string model_name,CCurrency* ca,CCurrency* cb ,ENUM_TIMEFRAMES tf_a,ENUM_TIMEFRAMES tf_b)
//+------------------------------------------------------------------+
  {

   m_ca = ca;
   m_cb = cb;
	m_timeframe_a=tf_a;
	m_timeframe_b=tf_b;
	m_model_name=model_name;
   return(0);
  }

//+------------------------------------------------------------------+
//|   Main function of the expert                                    |
//+------------------------------------------------------------------+
void CCurrencyModel::OnTick()
//+------------------------------------------------------------------+
  {
   //--- Recalculate the parameters of each tick
   if(SymbolInfoTick(m_ca.symbol,m_ca.tick))
     {
      m_ca.CalcHigh = 0;
      m_ca.CalcLow  = 0;
      double point  = SymbolInfoDouble(m_ca.symbol,SYMBOL_POINT);
      int    digits = (int)SymbolInfoInteger(m_ca.symbol,SYMBOL_DIGITS);

      //--- Check availability of historical data for the timeframe
      if(Bars(m_ca.symbol,m_timeframe_a)<PERIOD)
        {
         printf("Little historical data for trade");
         return;
        }

      else//--- Retrieving array data High and Low
      if(CopyHigh(m_ca.symbol,m_timeframe_a,0,PERIOD,m_ca.High)==PERIOD && CopyLow(m_ca.symbol,m_timeframe_a,0,PERIOD,m_ca.Low)==PERIOD)
        {
         m_ca.CalcHigh = m_ca.High[0];
         m_ca.CalcLow  = m_ca.Low[0];

         //--- Finding the minimum and maximum
         for(int j=1; j<PERIOD; j++)
           {
            if(m_ca.CalcHigh < m_ca.High[j]) m_ca.CalcHigh = m_ca.High[j];
            if(m_ca.CalcLow  >  m_ca.Low[j]) m_ca.CalcLow  = m_ca.Low[j];
           }
        }

      //--- Exit, if CalcHigh or CalcLow not counted
      if(m_ca.CalcHigh<0.01 || m_ca.CalcLow<0.01) return;

      //--- Update parameters
      m_ca.stop_level=SymbolInfoInteger(m_ca.symbol,SYMBOL_TRADE_STOPS_LEVEL);
      m_ca.spread=m_ca.tick.ask-m_ca.tick.bid;
      if(m_ca.order_step<m_ca.stop_level) m_ca.order_step=m_ca.stop_level;
      if(m_ca.trailing_level<m_ca.stop_level) m_ca.trailing_level=m_ca.stop_level;

      //--- The conditions of return inside the channel
      if(m_ca.tick.bid <= NormalizeDouble(m_ca.CalcHigh - m_ca.inside_level * point,digits)) m_ca.buy_open  = true;
      if(m_ca.tick.bid >= NormalizeDouble(m_ca.CalcLow  + m_ca.inside_level * point,digits)) m_ca.sell_open = true;

      //--- Processing of open orders
      WorkWithPositions(m_ca);

      //--- Processing of pending orders
      WorkWithPendingOrders(m_ca);
/*
      //--- Set a new pending order to buy
      if(m_ca.buy_open){
         OpenOrderBuyStop(m_ca);
      }

      //--- Set a new pending order to sell
      if(m_ca.sell_open){
         OpenOrderSellStop(m_ca);
      }
*/
     }
   else Print("Failed to retrieve data SymbolInfoTick(), error code: ",GetLastError());

   //--- Recalculate the parameters of each tick
   if(SymbolInfoTick(m_cb.symbol,m_cb.tick))
     {
      m_cb.CalcHigh = 0;
      m_cb.CalcLow  = 0;
      double point  = SymbolInfoDouble(m_cb.symbol,SYMBOL_POINT);
      int    digits = (int)SymbolInfoInteger(m_cb.symbol,SYMBOL_DIGITS);

      //--- Check availability of historical data for the timeframe
      if(Bars(m_cb.symbol,m_timeframe_b)<PERIOD)
        {
         printf("Little historical data for trade");
         return;
        }

      else//--- Retrieving array data High and Low
      if(CopyHigh(m_cb.symbol,m_timeframe_b,0,PERIOD,m_cb.High)==PERIOD && CopyLow(m_cb.symbol,m_timeframe_b,0,PERIOD,m_cb.Low)==PERIOD)
        {
         m_cb.CalcHigh = m_cb.High[0];
         m_cb.CalcLow  = m_cb.Low[0];

         //--- Finding the minimum and maximum
         for(int j=1; j<PERIOD; j++)
           {
            if(m_cb.CalcHigh < m_cb.High[j]) m_cb.CalcHigh = m_cb.High[j];
            if(m_cb.CalcLow  >  m_cb.Low[j]) m_cb.CalcLow  = m_cb.Low[j];
           }
        }

      //--- Exit, if CalcHigh or CalcLow not counted
      if(m_cb.CalcHigh<0.01 || m_cb.CalcLow<0.01) return;

      //--- Update parameters
      m_cb.stop_level=SymbolInfoInteger(m_cb.symbol,SYMBOL_TRADE_STOPS_LEVEL);
      m_cb.spread=m_cb.tick.ask-m_cb.tick.bid;
      if(m_cb.order_step<m_cb.stop_level) m_cb.order_step=m_cb.stop_level;
      if(m_cb.trailing_level<m_cb.stop_level) m_cb.trailing_level=m_cb.stop_level;

      //--- The conditions of return inside the channel
      if(m_cb.tick.bid <= NormalizeDouble(m_cb.CalcHigh - m_cb.inside_level * point,digits)) m_cb.buy_open  = true;
      if(m_cb.tick.bid >= NormalizeDouble(m_cb.CalcLow  + m_cb.inside_level * point,digits)) m_cb.sell_open = true;
/*
      //--- Processing of open orders
      WorkWithPositions(m_cb);

      //--- Processing of pending orders
      WorkWithPendingOrders(m_cb);

      //--- Set a new pending order to buy
      if(m_cb.buy_open){
         OpenOrderBuyStop(m_cb);
      }

      //--- Set a new pending order to sell
      if(m_cb.sell_open){
         OpenOrderSellStop(m_cb);
      }
*/
     }
   else 
      Print("Failed to retrieve data SymbolInfoTick(), error code: ",GetLastError());
   

   //--- Set a new pending order to buy
   if(m_ca.buy_open&&m_cb.sell_open){
      OpenOrderBuyStop(m_ca);
      OpenOrderSellStop(m_cb);
      //m_cb.sell_open = false;
   }
   
   //--- Set a new pending order to sell
   if(m_ca.sell_open&&m_cb.buy_open){
      OpenOrderSellStop(m_ca);
      OpenOrderBuyStop(m_cb);
      //m_cb.buy_open  =false;
   }


  }

//+------------------------------------------------------------------+
//|   Working with the positions                                     |
//+------------------------------------------------------------------+
void CCurrencyModel::WorkWithPositions(CCurrency* c)
//+------------------------------------------------------------------+
  {
   double point  = SymbolInfoDouble(c.symbol,SYMBOL_POINT);
   int    digits = (int)SymbolInfoInteger(c.symbol,SYMBOL_DIGITS);
   for(int pos=0; pos<PositionsTotal(); pos++)
     {

      if(PositionSelect(PositionGetSymbol(pos)))
        {
         if(PositionGetInteger(POSITION_MAGIC)!=c.magic || PositionGetString(POSITION_SYMBOL)!=c.symbol) continue;

         c.order_open_price = PositionGetDouble(POSITION_PRICE_OPEN);
         c.order_type       = PositionGetInteger(POSITION_TYPE);
         request.order    = PositionGetInteger(POSITION_IDENTIFIER);

         if(c.order_type==POSITION_TYPE_BUY)
           {
            c.buy_open=false;

            //--- Trailing stop
            if(c.stop_loss==0 || c.trailing_level==0)continue; 

            double sl=PositionGetDouble(POSITION_SL);
            double profit=PositionGetDouble(POSITION_PROFIT);
            double target=NormalizeDouble(MathMax(c.order_open_price,sl)+c.trailing_level*point,digits);

            if(c.tick.bid>target)
              {
               request.action = TRADE_ACTION_SLTP;
               request.symbol = c.symbol;
               request.sl     = MathMax(c.order_open_price,sl) + c.trailing_step * point;
               request.tp     = PositionGetDouble(POSITION_TP);
               request.deviation=c.slippage;

               OrderSend(request,result);
               if(result.retcode==10009 || result.retcode==10008) // 
                  if( VERBOSE) Print("Moving stop loss order Buy #",request.order);
               else
                  if( VERBOSE) Print("Error moving stop loss order Buy #",request.order);

              }
            continue;
           }// end POSITION_TYPE_BUY    

         else if(c.order_type==POSITION_TYPE_SELL)
           {
            c.sell_open=false;

            //--- Trailing stop
            if(c.stop_loss==0 || c.trailing_level==0) continue;

            double sl=PositionGetDouble(POSITION_SL);
            double profit=PositionGetDouble(POSITION_PROFIT);
            double target=NormalizeDouble(MathMin(c.order_open_price,sl)-c.trailing_level*point,digits);

            if(c.tick.ask<target)
              {
               request.action = TRADE_ACTION_SLTP;
               request.symbol = c.symbol;
               request.sl     = MathMin(c.order_open_price,sl) - c.trailing_step * point;
               request.tp     = PositionGetDouble(POSITION_TP);
               request.deviation=c.slippage;

               OrderSend(request,result);
               if(result.retcode==10009 || result.retcode==10008) // 
                  if( VERBOSE) Print("Moving stop loss order Sell #",request.order);
               else
                  if( VERBOSE) Print("Error moving stop loss order Sell #",request.order);
              }

           }// end POSITION_TYPE_SELL

        }// end if select                

     }// end for
  }

//+------------------------------------------------------------------+
//|   Working with pending orders                                    |
//+------------------------------------------------------------------+
void CCurrencyModel::WorkWithPendingOrders(CCurrency* c)
//+------------------------------------------------------------------+
  {
   double point  = SymbolInfoDouble(c.symbol,SYMBOL_POINT);
   int    digits = (int)SymbolInfoInteger(c.symbol,SYMBOL_DIGITS);
   //--- Processing pending orders
   for(int pos=0; pos<OrdersTotal(); pos++)
     {
      c.order_ticket=OrderGetTicket(pos);

      if(OrderSelect(c.order_ticket))
        {

         if(OrderGetInteger(ORDER_MAGIC)!=c.magic || OrderGetString(ORDER_SYMBOL)!=c.symbol) continue;

         c.order_type=OrderGetInteger(ORDER_TYPE);
         c.order_open_price=OrderGetDouble(ORDER_PRICE_OPEN);

         if(c.order_type==ORDER_TYPE_BUY_STOP)
           {
            c.buy_open=false;

            //--- Conditions for the movement of a pending order
            if(( NormalizeDouble(c.CalcHigh-c.order_step*point+c.spread,digits)<NormalizeDouble(c.order_open_price-c.order_step*point+c.spread,digits)) && 
               (NormalizeDouble(c.CalcHigh-c.order_step*point+c.spread,digits)>c.tick.ask+c.stop_level*point))
              {
               request.price=NormalizeDouble(c.CalcHigh-c.order_step*point+c.spread,digits);

               if(c.stop_loss==0) request.sl=0;
               else request.sl=NormalizeDouble(c.CalcHigh-c.order_step*point-MathMax(c.stop_loss,c.stop_level)*point,digits);

               if(c.take_profit==0) request.tp=0;
               else request.tp=NormalizeDouble(c.CalcHigh-c.order_step*point+MathMax(c.take_profit,c.stop_level)*point,digits);

               request.action     = TRADE_ACTION_MODIFY;
               request.order      = c.order_ticket;
               request.type_time  = ORDER_TIME_GTC;
               request.expiration = 0;

               OrderSend(request,result);
               if(result.retcode==10009 || result.retcode==10008)

                  if( VERBOSE) Print("Modified order BuyStop #",result.order,
                        StringFormat(" CalcLow=%1.5f order_step=%1.5f order_open_price=%1.5f bid=%1.5f stop_level=%1.5f",
                        c.CalcLow,c.order_step*point,c.order_open_price,c.tick.bid,c.stop_level*point));

               else
                  if( VERBOSE) Print("Error modified order BuyStop #",result.order,
                        StringFormat(" CalcLow=%1.5f order_step=%1.5f order_open_price=%1.5f bid=%1.5f stop_level=%1.5f err=%d",
                        c.CalcLow,c.order_step*point,c.order_open_price,c.tick.bid,c.stop_level*point,result.retcode));

              }
           }//end ORDER_TYPE_BUY_STOP  

         else if(c.order_type==ORDER_TYPE_SELL_STOP)
           {
            c.sell_open=false;

            //--- Conditions for the movement of a pending order
            if( NormalizeDouble(c.CalcLow+c.order_step*point,digits)>c.order_open_price+c.order_step*point && 
                NormalizeDouble(c.CalcLow+c.order_step*point,digits)<c.tick.bid-c.stop_level*point)
              {
               request.price=NormalizeDouble(c.CalcLow+c.order_step*point,digits);

               if(c.stop_loss==0) request.sl=0;
               else request.sl=NormalizeDouble(request.price+MathMax(c.stop_loss*point,c.stop_level*point)+c.spread,digits);

               if(c.take_profit==0) request.tp=0;
               else request.tp=NormalizeDouble(request.price-MathMax(c.take_profit*point,c.stop_level*point)+c.spread,digits);

               request.action    = TRADE_ACTION_MODIFY;
               request.order     = c.order_ticket;
               request.type_time = ORDER_TIME_GTC;

               OrderSend(request,result);

               if(result.retcode==10009 || result.retcode==10008)

                  if( VERBOSE) Print("Modified order SellStop #",result.order,
                        StringFormat(" CalcLow=%1.5f order_step=%1.5f order_open_price=%1.5f bid=%1.5f stop_level=%1.5f",
                        c.CalcLow,c.order_step*point,c.order_open_price,c.tick.bid,c.stop_level*point));
               else

                  if( VERBOSE) Print("Error modified order SellStop #",result.order,
                        StringFormat(" CalcLow=%1.5f order_step=%1.5f order_open_price=%1.5f bid=%1.5f stop_level=%1.5f",
                        c.CalcLow,c.order_step*point,c.order_open_price,c.tick.bid,c.stop_level*point));
              }

           }//end ORDER_TYPE_SELL_STOP   

        }// end order_ticket

     }// end for
  }
  
//+------------------------------------------------------------------+
//|   Set the pending order to buy                                   |
//+------------------------------------------------------------------+
void CCurrencyModel::OpenOrderBuyStop(CCurrency* c)
//+------------------------------------------------------------------+
  {
   double point  = SymbolInfoDouble(c.symbol,SYMBOL_POINT);
   int    digits = (int)SymbolInfoInteger(c.symbol,SYMBOL_DIGITS);
  
   c.lot=Calculate_Lot(c, LOT,LOT_TYPE,ORDER_TYPE_BUY); //Do not specify ORDER_TYPE_BUY_STOP.

   request.price=NormalizeDouble(c.CalcHigh-c.order_step*point+c.spread,digits);

   //--- Adjust the price of orders
   if(c.tick.ask+c.stop_level*point>request.price) request.price=c.tick.ask+c.stop_level*point;

   //--- Calculation of stop loss
   if(c.stop_loss==0) request.sl=0;
   else request.sl=NormalizeDouble(request.price-MathMax(c.stop_loss,c.stop_level)*point-c.spread,digits);

   //--- Calculation of take profit
   if(c.take_profit==0) request.tp=0;
   else request.tp=NormalizeDouble(request.price+MathMax(c.take_profit,c.stop_level)*point-c.spread,digits);

//request.tp=0.0;
//request.sl=0.0;
   request.action       = TRADE_ACTION_PENDING;//TRADE_ACTION_DEAL
   request.symbol       = c.symbol;
   request.volume       = c.lot;
   request.deviation    = c.slippage;
   request.type         = ORDER_TYPE_BUY_STOP;//ORDER_TYPE_BUY
   request.type_filling = ORDER_FILLING_AON;
   request.type_time    = ORDER_TIME_GTC;
   request.comment      = IntegerToString(c.magic);
   request.magic        = c.magic;

   OrderSend(request,result);
   if(result.retcode==10009 || result.retcode==10008)
     {
      c.buy_open=false;
      if( VERBOSE) Print("Opened an order BuyStop #",result.order);
     }
   else
      if( VERBOSE) printf("Error opened an order BuyStop. Error code: %d",result.retcode);

  }
  
//+------------------------------------------------------------------+
//|   Set the pending order to sell                                  |
//+------------------------------------------------------------------+
void CCurrencyModel::OpenOrderSellStop(CCurrency* c)
//+------------------------------------------------------------------+
  {
   double point  = SymbolInfoDouble(c.symbol,SYMBOL_POINT);
   int    digits = (int)SymbolInfoInteger(c.symbol,SYMBOL_DIGITS);

   c.lot=Calculate_Lot(c, LOT,LOT_TYPE,ORDER_TYPE_SELL);// Do not specify ORDER_TYPE_SELL_STOP.

   request.price=NormalizeDouble(c.CalcLow+c.order_step*point,digits);

   //--- adjust the price of orders
   if(c.tick.bid-c.stop_level*point<request.price) request.price=c.tick.bid-c.stop_level*point;

   //--- The calculation of stop loss
   if(c.stop_loss==0) request.sl=0;
   else request.sl=NormalizeDouble(request.price+MathMax(c.stop_loss*point,c.stop_level*point)+c.spread,digits);

   //--- The calculation of take profit
   if(c.take_profit==0) request.tp=0;
   else request.tp=NormalizeDouble(request.price-MathMax(c.take_profit*point,c.stop_level*point)+c.spread,digits);

//request.tp=0.0;
//request.sl=0.0;
   request.action       = TRADE_ACTION_PENDING;//TRADE_ACTION_DEAL
   request.symbol       = c.symbol;
   request.volume       = c.lot;
   request.deviation    = c.slippage;
   request.type         = ORDER_TYPE_SELL_STOP;//ORDER_TYPE_SELL
   request.type_filling = ORDER_FILLING_AON;
   request.type_time    = ORDER_TIME_GTC;
   request.comment      = IntegerToString(c.magic);
   request.magic        = c.magic;

   OrderSend(request,result);

   if(result.retcode==10009 || result.retcode==10008) // 
     {
      c.sell_open=false;
      if( VERBOSE) Print("Opened an order SellStop #",result.order);
     }
   else
      if( VERBOSE) printf("Error opened an order SellStop, error code: %d", result.order);

  }
//+------------------------------------------------------------------+
//|   Calculation of the lot                                         |
//+------------------------------------------------------------------+
double CCurrencyModel::Calculate_Lot(CCurrency* c, double lot_value,int type,ENUM_ORDER_TYPE direction)
//+------------------------------------------------------------------+
  {
   double acc_free_margin=AccountInfoDouble(ACCOUNT_FREEMARGIN);
   double calc_margin;
   double price=0;

   if(direction == ORDER_TYPE_BUY)  price = c.tick.ask;
   if(direction == ORDER_TYPE_SELL) price = c.tick.bid;

   switch(type)
     {
      case e_fixed:
        {
         //--- Correction of lot size
         if(LOT_CORRECTION)
           {
            OrderCalcMargin(direction,c.symbol,lot_value,price,calc_margin);
            //--- Lot size correction of up to 90% of free margin
            if(acc_free_margin<calc_margin)
              {
               lot_value=lot_value*acc_free_margin*0.9/calc_margin;
               if( VERBOSE) printf("Adjusted value of the lot: %f",c.lot);
              }
           }
         break;
        }

      case e_percent:
        {
         //--- value of free margin for open position
         OrderCalcMargin(direction,c.symbol,1,price,calc_margin);
         if( VERBOSE) printf("Adjusted value of the lot: %f",price);
         if( calc_margin == 0.0 )
            calc_margin = 0.01;
         lot_value=acc_free_margin*0.01*LOT/calc_margin;
         break;
        }
     }// end switch

   return(NormalizeLot(c,lot_value));
  }
  
//+------------------------------------------------------------------+
//|   Normalization of the lot size                                  |
//+------------------------------------------------------------------+
double CCurrencyModel::NormalizeLot(CCurrency* c, double lot_value)
//+------------------------------------------------------------------+
  {
   double lot_min  = SymbolInfoDouble(c.symbol, SYMBOL_VOLUME_MIN);
   double lot_max  = SymbolInfoDouble(c.symbol, SYMBOL_VOLUME_MAX);
   double lot_step = SymbolInfoDouble(c.symbol, SYMBOL_VOLUME_STEP);
   int norm;

   if(lot_value <= lot_min ) lot_value = lot_min;               // checking for minimum lot
   else if(lot_value >= lot_max ) lot_value = lot_max;          // checking the maximum lot
   else(lot_value = MathFloor(lot_value/ lot_step) * lot_step); // rounding to the nearest

   norm=(int)NormalizeDouble(MathCeil(MathLog10(1/lot_step)),0);// coefficient for NormalizeDouble
   lot_value = (NormalizeDouble(lot_value,norm));                     // normalization
   if( lot_value > 5.0 )
      lot_value = 5.0;
   return lot_value;   
   }
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
	//ExtExpert.OnInit();
	EventSetTimer(300);    
	gCurrencyModelList = new CList;
	
	
	CCurrencyModel *eurUsdModel;
	gEurUsd = new CCurrency("EURUSD",12345);
	gUsdChf = new CCurrency("USDCHF",54321);
	eurUsdModel = new CCurrencyModel();
	if( eurUsdModel.OnInit("EURUSD versus USDCHF model",gEurUsd,gUsdChf,PERIOD_M15,PERIOD_M15) == 0 ){
        gCurrencyModelList.Add(eurUsdModel);
   } else {
         // initialization of currency has failed
        Print("Model ",eurUsdModel.ModelName()," with period = ",eurUsdModel.PeriodA(),
               " on symbol ",eurUsdModel.SymbolA()," create has failed");
   }
/*
	CCurrencyModel *gbpUsdModel;
	gGbpUsd = new CCurrency("GBPUSD",67890);
	gEurGbp = new CCurrency("EURGBP",09876);
//	gGbpUsd = new CCurrency("AUDUSD",67890);
//	gEurGbp = new CCurrency("EURAUD",09876);
	gbpUsdModel = new CCurrencyModel();
	if( gbpUsdModel.OnInit("AUDUSD versus EURAUD model",gGbpUsd,gEurGbp,PERIOD_M15,PERIOD_M15) == 0 ){
        gCurrencyModelList.Add(gbpUsdModel);
   } else {
         // initialization of currency has failed
        Print("Model ",gbpUsdModel.ModelName()," with period = ",gbpUsdModel.PeriodA(),
               " on symbol ",gbpUsdModel.SymbolA()," create has failed");
   }

	CCurrencyModel *audUsdModel;
	gAudUsd = new CCurrency("AUDUSD",34567);
	gEurAud = new CCurrency("EURAUD",76543);
	audUsdModel = new CCurrencyModel();
	if( audUsdModel.OnInit("AUDUSD versus EURAUD model",gAudUsd,gEurAud,PERIOD_M15,PERIOD_M15) == 0 ){
        gCurrencyModelList.Add(audUsdModel);
   } else {
         // initialization of currency has failed
        Print("Model ",audUsdModel.ModelName()," with period = ",audUsdModel.PeriodA(),
               " on symbol ",audUsdModel.SymbolA()," create has failed");
   }
  */ 
//--- ok
   return(0);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   //ExtExpert.Deinit();
   delete gCurrencyModelList;
   delete gGbpUsd;
   delete gEurGbp;
   delete gEurUsd;
   delete gUsdChf;
   delete gEurAud;
   delete gAudUsd;   
   
  }
//+------------------------------------------------------------------+
//| Function-event handler "tick"                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
	//ExtExpert.OnTick();
	/*
	CCurrencyModel *cm;                         // create pointer to base currency
	for(int i=0;i<gCurrencyModelList.Total();i++)
     { // proceed all currencies in the list
      cm=gCurrencyModelList.GetNodeAtIndex(i); // set pointer to the current currency
      cm.OnTick();                 // call processing of the current currency
     }
     */
  }
//+------------------------------------------------------------------+
//| Function-event handler "trade"                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   //ExtExpert.OnTrade();
  }
//+------------------------------------------------------------------+
//| Function-event handler "timer"                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
  //ExtExpert.OnTick();
	CCurrencyModel *cm;                         // create pointer to base currency
	for(int i=0;i<gCurrencyModelList.Total();i++)
     { // proceed all currencies in the list
      cm=gCurrencyModelList.GetNodeAtIndex(i); // set pointer to the current currency
      cm.OnTick();                 // call processing of the current currency
     }
  }
//+------------------------------------------------------------------+
