//+------------------------------------------------------------------+
//|                                                       perdon     |
//+------------------------------------------------------------------+
// Version 4_SL - absolute value of TP
#property version   "4.00"

#include <Trade\Trade.mqh>
#include <Indicators\Trend.mqh>
#include <Trade\PositionInfo.mqh>
#include <Trade\AccountInfo.mqh>
#include <Trade\SymbolInfo.mqh>
#include <Trade\OrderInfo.mqh>

CTrade        myTrade;
CPositionInfo myPosition;
CAccountInfo  myAccountInfo;
CSymbolInfo   mySymbol;
COrderInfo myorder;
input double      TP = 800.;
input double      kSL = 1.1;
input int         perDon = 6;
input double      kBalance=1;

double maxD, minD;
MqlDateTime TimeStruct;
datetime TimeWorkBar = 0;
bool Buy_opened=false,Sell_opened=false;
MqlRates    mrate[];


double minVolume=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MIN);
double maxVolume=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_MAX);
double volumeStep=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_STEP);
double limVolume=SymbolInfoDouble(Symbol(),SYMBOL_VOLUME_LIMIT);


//+------------------------------------------------------------------+
//| Calculate the volume of the next trade                           |
//+------------------------------------------------------------------+
double Lots(double k, int bunch = 0)
  {

   double volume=round(k/(10000*volumeStep)*myAccountInfo.Balance())*volumeStep;
   
   volume = volume - maxVolume*bunch;
   
   if(volume < 0) {
       volume = 0;
   } else if (volume<minVolume) {
      volume=minVolume;
   } else if (volume>maxVolume) {
      volume=maxVolume;
   }
    
   return(volume);
   
  }

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
{
//---
   mySymbol.Name(_Symbol);
   return(0);
}
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
{
//---
}

void OnNewBar()
{    
      minD=mrate[0].low; maxD=mrate[0].high;
      for (int i=1; i<=perDon; i++) {
         minD = MathMin(mrate[i].low,minD);
         maxD = MathMax(mrate[i].high,maxD);
      }

      Buy_opened=false;Sell_opened=false;
      if(PositionSelect(_Symbol)) {
        if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_BUY) {
            Buy_opened=true;
        } else if(PositionGetInteger(POSITION_TYPE)==POSITION_TYPE_SELL) {
            Sell_opened=true;
        }
      }
 
     uint     total=OrdersTotal();
     ulong    ticket=0,ticketBuyLimit=0,ticketSellLimit=0;
//    Loop over all orders
     for(uint i=0;i<total;i++)
     {
      // Lets get the order ticket and its position in the list
         if((ticket=OrderGetTicket(i))>0) {
            string symbol      =OrderGetString(ORDER_SYMBOL);
            long type          =OrderGetInteger(ORDER_TYPE);
            if (symbol == _Symbol) {
               if (type == ORDER_TYPE_BUY_LIMIT || type == ORDER_TYPE_SELL_LIMIT) {
                  myTrade.OrderDelete(ticket);
               }
            }
         }
     }


      double dist = fabs(maxD - minD);
      double entryBuy = minD;
      double entrySell = maxD;

      double tpSell =  Round5(entrySell - TP*Point());
      double slSell = Round5(entrySell + kSL*dist);

      double tpBuy  = Round5(entryBuy + TP*Point());
      double slBuy = Round5(entryBuy - kSL*dist);

      double posVolume=0;
      if(myPosition.Select(_Symbol)==true) {
         posVolume=myPosition.Volume();
      }
      
      if(!Buy_opened) {//Buy position

         bool moneyWise = myAccountInfo.Margin() < 0.5 * myAccountInfo.Balance();
         for (int volIdx = 0; volIdx < 1; volIdx++) {  // Loop is not active at the moment
            double volume = Lots(kBalance, volIdx);
            if (volume > 0 && volume + posVolume + OrdersTotal()*maxVolume <= limVolume && moneyWise) {
               myTrade.BuyLimit(volume,entryBuy, NULL, slBuy, tpBuy);
               myTrade.SellLimit(volume ,entrySell, NULL, slSell, tpSell);
            }
         }

      }
}

//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
{
//---
      TimeToStruct(TimeCurrent(),TimeStruct);
      ArraySetAsSeries(mrate,true);
      if (!mySymbol.RefreshRates())
      {
         Alert("Error mySymbol.RefreshRates: ",GetLastError(),"!");
         return;
      }
      if(CopyRates(_Symbol,_Period,0,perDon+2,mrate)<0)
      {
         Alert("CopyRates error: ",GetLastError(),"!!");
         return;
      }
      datetime timeCurrentBar=mrate[0].time;
      if (timeCurrentBar>TimeWorkBar)
      {
         OnNewBar();

         TimeWorkBar = timeCurrentBar;
      }
}
//+------------------------------------------------------------------+


//+------------------------------------------------------------------+
//| Round the price for EURUSD (5 digits after point)                |
//+------------------------------------------------------------------+

double Round5(double val)
{
   return round(val*100000)/100000;     
}